﻿using System;
using System.Collections.Generic;
using System.IO;
using System.Linq;
using BepInEx;
using BepInEx.Logging;
using HarmonyLib;
using KSP.Game;
using UnityEngine;
using UnityEngine.AddressableAssets;
using UnityEngine.AddressableAssets.ResourceLocators;
using UnityEngine.ResourceManagement.ResourceProviders;

namespace CustomFlagLoader
{
    [BepInPlugin("io.bepis.customflagloader", "Custom Flag Loader", "1.0")]
    public class CustomFlagLoader : BaseUnityPlugin
    {
        private static string FlagDirectory { get; set; }

        private static ManualLogSource Logger { get; set; }

        private static bool HasInitialized { get; set; } = false;
        private static CustomResourceProvider ResourceProvider { get; } = new CustomResourceProvider();

        public CustomFlagLoader()
        {
            FlagDirectory = Path.Combine(Paths.GameRootPath, "flags");
            Logger = base.Logger;

            Directory.CreateDirectory(FlagDirectory);

            Harmony.CreateAndPatchAll(typeof(CustomFlagLoader), "io.bepis.customflagloader");
        }

        [HarmonyPrefix, HarmonyPatch(typeof(AgencyFlagManager), "InitializeFlags")]
        public static void InitializeFlagsPrefix(AgencyFlagsData ____agencyFlags)
        {
            if (HasInitialized)
                return;

            Logger.LogDebug("Loading flags via InitializeFlags");

            var flagNames = new List<string>();

            foreach (var flagPath in Directory.GetFiles(FlagDirectory, "*.png"))
            {
                Logger.LogDebug($"Loading {Path.GetFileName(flagPath)}");

                var flagName = "custom-flag-" + Path.GetFileNameWithoutExtension(flagPath);

                var flagTexture = new Texture2D(1, 1); 
                flagTexture.LoadImage(File.ReadAllBytes(flagPath));

                var flagRect = new Rect(0, 0, flagTexture.width, flagTexture.height);
                var flagSprite = Sprite.Create(flagTexture, flagRect,
                    new Vector2(flagTexture.width / 2f, flagTexture.height / 2f)); // i'm assuming the pivot is in the center
                flagSprite.name = flagName;


                ____agencyFlags.Flags.Add(new AgencyFlag
                {
                    FlagIcon = flagSprite,
                    FlagName = flagName,
                    PrimaryColor = Color.white,
                    SecondaryColor = Color.black
                });

                flagNames.Add(flagName);
                ResourceProvider.Resources.Add(flagName, new Tuple<Texture2D, Sprite>(flagTexture, flagSprite));
            }

            Addressables.AddResourceLocator(new ResourceLocationMap("io.bepis.customflagloader-sprite", flagNames
                .Select(x => new ResourceLocationData(new[] { x }, x, typeof(CustomResourceProvider), typeof(Sprite))).ToList()));

            Addressables.AddResourceLocator(new ResourceLocationMap("io.bepis.customflagloader-tx2d", flagNames
                .Select(x => new ResourceLocationData(new[] { x }, x, typeof(CustomResourceProvider), typeof(Texture2D))).ToList()));

            Addressables.ResourceManager.ResourceProviders.Add(ResourceProvider);

            Logger.LogDebug("Finished loading flags");

            HasInitialized = true;
        }


        public class CustomResourceProvider : ResourceProviderBase
        {
            public Dictionary<string, Tuple<Texture2D, Sprite>> Resources { get; } =
                new Dictionary<string, Tuple<Texture2D, Sprite>>();

            public override void Provide(ProvideHandle provideHandle)
            {
                bool success = true;
                
                if (Resources.TryGetValue(provideHandle.Location.PrimaryKey, out var resource))
                {
                    if (provideHandle.Location.ResourceType == typeof(Texture2D))
                        provideHandle.Complete(resource.Item1, true, null);
                    else if (provideHandle.Location.ResourceType == typeof(Sprite))
                        provideHandle.Complete(resource.Item2, true, null);
                    else
                        success = false;
                }
                else
                {
                    success = false;
                }

                if (!success)
                    provideHandle.Complete<object>(null, false, null);
            }
        }
    }
}
